@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2011 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2014 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -27,6 +27,7 @@
 #include "wx/wfstream.h"
 #include "wx/dir.h"
 #include "wx/filename.h"
+#include <wx/numdlg.h>
 
 #include "speak_lib.h"
 #include "main.h"
@@ -43,8 +44,10 @@
 
 extern void FindPhonemesUsed(void);
 extern void DisplayErrorFile(const char *fname);
+extern int utf8_in(int *c, const char *buf);
 extern int utf8_out(unsigned int c, char *buf);
 extern void DrawEnvelopes();
+extern void ReadPhondataManifest();
 char path_source[sizeof(path_home)+20];
 
 
@@ -71,6 +74,7 @@ typedef struct {
 #define k_AND     1
 #define k_OR      2
 #define k_THEN    3
+#define k_NOT     4
 
 #define kTHISSTRESS 0x800
 
@@ -90,17 +94,18 @@ static keywtab_t k_conditions[] = {
 	{"AND",      0,  k_AND},
 	{"OR",       0,  k_OR},
 	{"THEN",     0,  k_THEN},
-	{"prevPh",   tWHICH_PHONEME,  0x000},
-	{"prevPhW",  tWHICH_PHONEME,  0x500},
-	{"thisPh",   tWHICH_PHONEME,  0x100},
-	{"nextPh",   tWHICH_PHONEME,  0x200},
-	{"next2Ph",  tWHICH_PHONEME,  0x300},
-	{"nextPhW",  tWHICH_PHONEME,  0x400},
-	{"nextVowel",tWHICH_PHONEME,  0x600},
-//	{"next2PhW", tWHICH_PHONEME,  0x700},
-
-//	{"numVowels",    tTEST,  0x000},
-//	{"afterStress",  tTEST,  0x000},
+	{"NOT",      0,  k_NOT},
+	{"prevPh",   tWHICH_PHONEME,  0},
+	{"prevPhW",  tWHICH_PHONEME,  5},
+	{"thisPh",   tWHICH_PHONEME,  1},
+	{"nextPh",   tWHICH_PHONEME,  2},
+	{"next2Ph",  tWHICH_PHONEME,  3},
+	{"nextPhW",  tWHICH_PHONEME,  4},
+	{"next2PhW",  tWHICH_PHONEME, 6},
+	{"nextVowel",tWHICH_PHONEME,  7},
+	{"prevVowel",tWHICH_PHONEME,  8},
+	{"next3PhW", tWHICH_PHONEME,  9},
+	{"prev2PhW", tWHICH_PHONEME, 10},
 
 	{"PreVoicing",   tTEST,  0xf01},
 	{"KlattSynth",   tTEST,  0xf02},
@@ -119,6 +124,7 @@ static keywtab_t k_properties[] = {
 	{"isVFricative", 0,  phVFRICATIVE},
 
 	{"isPalatal",    0,  i_isPalatal},
+	{"isLong",       0,   i_isLong},
 	{"isRhotic",     0,  i_isRhotic},
 	{"isSibilant",   0,  i_isSibilant},
 	{"isFlag1",      0,  i_isFlag1},
@@ -152,6 +158,7 @@ static keywtab_t k_properties[] = {
 
 enum {
 	kPHONEMESTART = 1,
+	kUTF8_BOM,
 	kPROCEDURE,
 	kENDPHONEME,
 	kENDPROCEDURE,
@@ -205,6 +212,8 @@ enum {
 	kTUNE_SPLIT,
 };
 
+static const char utf8_bom[] = {0xef,0xbb,0xbf,0};
+
 static keywtab_t k_intonation[] = {
 	{"tune",      0,   kTUNE},
 	{"endtune",   0,   kENDTUNE},
@@ -245,6 +254,7 @@ static keywtab_t keywords[] = {
 	{"phonemenumber",tSTATEMENT, kPHONEMENUMBER},
 	{"phonemetable",tSTATEMENT, kPHONEMETABLE},
 	{"include",     tSTATEMENT, kINCLUDE},
+	{utf8_bom,      tSTATEMENT, kUTF8_BOM},
 
 	{"phoneme",    tSTATEMENT, kPHONEMESTART},
 	{"procedure",  tSTATEMENT, kPROCEDURE},
@@ -283,7 +293,7 @@ static keywtab_t keywords[] = {
 	{"Continue",   tSTATEMENT, kCONTINUE},
 
 	{"ChangePhoneme", tINSTRN1, i_CHANGE_PHONEME},
-	{"ReplaceNextPhoneme", tINSTRN1, i_REPLACE_NEXT_PHONEME},
+	{"ChangeNextPhoneme", tINSTRN1, i_REPLACE_NEXT_PHONEME},
 	{"InsertPhoneme", tINSTRN1, i_INSERT_PHONEME},
 	{"AppendPhoneme", tINSTRN1, i_APPEND_PHONEME},
 	{"IfNextVowelAppend", tINSTRN1, i_APPEND_IFNEXTVOWEL},
@@ -462,7 +472,7 @@ return;
 		instn_category = (instn >> 12) & 0xf;
 		data1 = instn & 0xff;
 		type2 = (instn >> 8) & 0xf;
-		fprintf(f_out, "  %.3x: %.4x  %s",pc-prog_buf,instn,instn_category_string[instn_category]);
+		fprintf(f_out, "  %.3x: %.4x  %s",(unsigned int)(pc-prog_buf),instn,instn_category_string[instn_category]);
 
 		switch(instn_category)
 		{
@@ -605,7 +615,7 @@ return;
 		case 13:
 		case 14:
 		case 15:
-			address = ((instn & 0xf) << 16) + *pc++; 
+			address = ((instn & 0xf) << 16) + *pc++;
 			fprintf(f_out, " %d %.5x",(instn >> 4) & 0xff,address*4);
 			break;
 		}
@@ -627,6 +637,7 @@ static FILE *f_phtab;
 static FILE *f_phcontents;
 static FILE *f_errors = stderr;
 static FILE *f_prog_log = NULL;
+static FILE *f_report;
 
 
 static FILE *f_in;
@@ -639,6 +650,8 @@ static int count_references = 0;
 static int duplicate_references = 0;
 static int count_frames = 0;
 static int error_count = 0;
+static int resample_count = 0;
+static int resample_fails = 0;
 static int then_count = 0;
 static int after_if = 0;
 
@@ -658,9 +671,9 @@ static REF_HASH_TAB *ref_hash_tab[256];
 
 
 #define N_ENVELOPES  30
-static int n_envelopes = 0;
-static char envelope_paths[N_ENVELOPES][80];
-static unsigned char envelope_dat[N_ENVELOPES][ENV_LEN];
+int n_envelopes = 0;
+char envelope_paths[N_ENVELOPES][80];
+unsigned char envelope_dat[N_ENVELOPES][ENV_LEN];
 
 
 typedef struct {
@@ -730,27 +743,21 @@ static void CompileReport(void)
 	int n;
 	REF_HASH_TAB *p;
 	REF_HASH_TAB **list;
-	FILE *f_report;
 	const char *data_path;
 	int prev_table;
+	int procedure_num;
 	int prev_mnemonic;
-	char fname[sizeof(path_source)+20];
+
+    if(f_report == NULL)
+        return;
 
 	// make a list of all the references and sort it
 	list = (REF_HASH_TAB **)malloc((count_references)* sizeof(REF_HASH_TAB *));
 	if(list == NULL)
 		return;
 
-	sprintf(fname,"%scompile_report",path_source);
-	f_report = fopen(fname,"w");
-	if(f_report == NULL)
-	{
-		free(list);
-		return;
-	}
-
-	fprintf(f_report,"%d phoneme tables\n",n_phoneme_tabs);
-   fprintf(f_report,"          new total\n");
+	fprintf(f_report,"\n%d phoneme tables\n",n_phoneme_tabs);
+    fprintf(f_report,"          new total\n");
 	for(ix=0; ix<n_phoneme_tabs; ix++)
 	{
 		fprintf(f_report,"%8s %3d %4d\n",phoneme_tab_list2[ix].name, phoneme_tab_list2[ix].n_phonemes, n_phcodes_list[ix]+1);
@@ -770,7 +777,7 @@ static void CompileReport(void)
 	}
 	n = ix;
 	qsort((void *)list,n,sizeof(REF_HASH_TAB *),(int (*)(const void *,const void *))ref_sorter);
-	
+
 	data_path = "";
 	prev_mnemonic = 0;
 	prev_table = 0;
@@ -796,7 +803,17 @@ static void CompileReport(void)
 			j++;
 		}
 
-		fprintf(f_report,"  [%s] %s",WordToString(prev_mnemonic = list[ix]->ph_mnemonic), phoneme_tab_list2[prev_table = list[ix]->ph_table].name);
+		prev_mnemonic = list[ix]->ph_mnemonic;
+		if((prev_mnemonic >> 24) == 'P')
+		{
+			// a procedure, not a phoneme
+			procedure_num = atoi(WordToString(prev_mnemonic));
+			fprintf(f_report,"  %s  %s", phoneme_tab_list2[prev_table = list[ix]->ph_table].name, proc_names[procedure_num]);
+		}
+		else
+		{
+			fprintf(f_report,"  [%s] %s",WordToString(prev_mnemonic), phoneme_tab_list2[prev_table = list[ix]->ph_table].name);
+		}
 		fputc('\n',f_report);
 	}
 
@@ -974,6 +991,10 @@ static wxString CompileAllDictionaries()
 
 	sprintf(fname_log,"%s%s",path_dsource,"dict_log");
 	log = fopen(fname_log,"w");
+	if(log != 0)
+	{
+		fprintf(log, "%s", utf8_bom);
+	}
 	sprintf(fname_log,"%s%s",path_dsource,"dict_phonemes");
 	f_phused = fopen(fname_log,"w");
 
@@ -1034,8 +1055,13 @@ static wxString CompileAllDictionaries()
 			fclose(f_in);
 		}
 
-		LoadVoice(voicename,0);
-
+		if(LoadVoice(voicename,1) == NULL)
+		{
+			wxLogError(wxString::Format(_T("Can't find voice '%s' for dictionary '%s'"), wxString(voicename, wxConvLocal).c_str(), dictstr.c_str()));
+			report = report + dictstr + _T(" No Voice, ");
+			errors ++;
+		}
+		else
 		if((err = CompileDictionary(path_dsource, dictname,log,NULL,0)) > 0)
 		{
 			report = report + dictstr + wxString::Format(_T(" %d, "),err);
@@ -1106,6 +1132,9 @@ static unsigned int StringToWord(const char *string)
 	unsigned char c;
 	unsigned int word;
 
+	if(string==NULL)
+		return(0);
+
 	word = 0;
 	for(ix=0; ix<4; ix++)
 	{
@@ -1197,7 +1226,7 @@ static int LookupPhoneme(const char *string, int control)
 
 	// don't use phoneme number 0, reserved for string terminator
 	start = 1;
-	
+
 	if(control==2)
 		start = 8;   // don't look for control and stress phonemes (allows these characters to be
 		             // used for other purposes)
@@ -1267,12 +1296,12 @@ static int NextItem(int type)
 	int  sign;
 	char *p;
 	keywtab_t *pk;
-	
+
 	item_type = -1;
 
 	f_in_displ = ftell(f_in);
 	f_in_linenum = linenum;
-		
+
 	while(!feof(f_in))
 	{
 		c = get_char();
@@ -1389,6 +1418,24 @@ static int NextItem(int type)
 
 
 
+static int NextItemMax(int max)
+{//==============================
+// Get a number, but restrict value to max
+    int value;
+    char msg[80];
+
+    value = NextItem(tNUMBER);
+    if(value > max)
+    {
+        sprintf(msg, "Value %d is greater than maximum %d", value, max);
+        error(msg, NULL);
+        value = max;
+    }
+    return(value);
+}
+
+
+
 static int NextItemBrackets(int type, int control)
 {//===============================================
 // Expect a parameter inside parantheses
@@ -1592,7 +1639,7 @@ int LoadSpect(const char *path, int control)
 
 	wxString filename = path_phsource + path_sep + wxString(path,wxConvLocal);
 	wxFileInputStream stream(filename);
-	
+
 	if(stream.Ok() == FALSE)
 	{
 		error("Failed to open: '%s'",path);
@@ -1600,6 +1647,12 @@ int LoadSpect(const char *path, int control)
 	}
 	spectseq->Load(stream);
 
+	if(spectseq->frames == NULL)
+	{
+		error("Bad vowel file, no frames: '%s'",path);
+		return(0);
+	}
+
 	// do we need additional klatt data ?
 	for(frame=0; frame < spectseq->numframes; frame++)
 	{
@@ -1616,7 +1669,7 @@ int LoadSpect(const char *path, int control)
 	seq_out.sqflags=0;
 	seq_out.length_total=0;
 
-	total = 0;	
+	total = 0;
 	for(frame=0; frame < spectseq->numframes; frame++)
 	{
 
@@ -1650,7 +1703,7 @@ for(ix=0; ix<8; ix++)
 			seq_out.n_frames++;
 			if(frame > 0)
 				total += spectseq->frames[frame-1]->length;
-		}	
+		}
 	}
 	seq_out.length_total = int(total);
 
@@ -1665,7 +1718,7 @@ for(ix=0; ix<8; ix++)
 	for(frame=0; frame < spectseq->numframes; frame++)
 	{
 		fr = spectseq->frames[frame];
-		
+
 		if(fr->keyframe)
 		{
 			if(klatt_flag)
@@ -1756,7 +1809,7 @@ for(ix=0; ix<8; ix++)
 			n_frames++;
 		}
 	}
-	
+
 	if(klatt_flag)
 	{
 		seqk_out.n_frames = seq_out.n_frames;
@@ -1792,53 +1845,82 @@ static int LoadWavefile(FILE *f, const char *fname)
 	int max = 0;
 	int length;
 	int sr1, sr2;
+	int failed;
+	int len;
 	int resample_wav = 0;
+	const char *fname2;
 	char fname_temp[100];
+	char msg[120];
 	int scale_factor=0;
 
 	fseek(f,24,SEEK_SET);
 	sr1 = Read4Bytes(f);
 	sr2 = Read4Bytes(f);
-	fseek(f,40,SEEK_SET);	
+	fseek(f,40,SEEK_SET);
 
 	if((sr1 != samplerate_native) || (sr2 != sr1*2))
 	{
-#ifdef PLATFORM_WINDOWS
-		if(sr1 != samplerate_native)
+        int fd_temp;
+        char command[sizeof(path_source)+200];
+
+	    failed = 0;
+
+#ifdef PLATFORM_POSIX
+		strcpy(fname_temp,"/tmp/espeakXXXXXX");
+		if((fd_temp = mkstemp(fname_temp)) >= 0)
 		{
-			fprintf(f_errors,"Wrong samplerate %d, wants %d\n",sr1,samplerate_native);
-			error("Wrong samplerate: %s",fname);
-		}
-		if(sr2 != sr1*2)
-		{
-			error("Not mono: %s",fname);
+			close(fd_temp);
 		}
 #else
-		{
-			int fd_temp;
-			char command[sizeof(path_source)+200];
-			strcpy(fname_temp,"/tmp/espeakXXXXXX");
-			if((fd_temp = mkstemp(fname_temp)) >= 0)
-			{
-				close(fd_temp);
-				sprintf(command,"sox \"%s%s.wav\" -r %d -c 1 -w  %s polyphase\n",path_source,fname,samplerate_native,fname_temp);
-				if(system(command) < 0)
-				{
-					error("Failed to resample: %s",command);
-					return(0);
-				}
-			}
-	
-			f = fopen(fname_temp,"rb");
-			if(f == NULL)
-			{
-				error("Can't read temp file: %s",fname_temp);
-				return(0);
-			}
-			resample_wav = 1;
-			fseek(f,40,SEEK_SET);  // skip past the WAV header, up to before "data length"
-		}
+		strcpy(fname_temp,tmpnam(NULL));
 #endif
+
+        fname2 = fname;
+        len = strlen(fname);
+        if(strcmp(&fname[len-4], ".wav") == 0)
+        {
+            strcpy(msg, fname);
+            msg[len-4] = 0;
+            fname2 = msg;
+        }
+
+        sprintf(command,"sox \"%s%s.wav\" -r %d -c1 -t wav %s\n",path_source,fname2,samplerate_native, fname_temp);
+        if(system(command) != 0)
+        {
+            failed = 1;
+        }
+
+
+        if(failed || (GetFileLength(fname_temp) <= 0))
+        {
+            if(resample_fails < 2)
+                error("Resample command failed: %s", command);
+            resample_fails++;
+
+            if(sr1 != samplerate_native)
+            {
+                sprintf(msg, "Can't resample (%d to %d): %s", sr1, samplerate_native, fname);
+                error("%s", msg);
+            }
+            else
+            {
+                error("WAV file is not mono: %s", fname);
+            }
+            remove(fname_temp);
+            return(0);
+        }
+
+        f = fopen(fname_temp,"rb");
+        if(f == NULL)
+        {
+            error("Can't read temp file: %s",fname_temp);
+            return(0);
+        }
+        if(f_report != NULL)
+            fprintf(f_report, "resampled  %s\n", fname);
+        resample_count++;
+        resample_wav = 1;
+        fseek(f,40,SEEK_SET);  // skip past the WAV header, up to before "data length"
 	}
 
 	displ = ftell(f_phdata);
@@ -2078,7 +2160,7 @@ static int LoadDataFile(const char *path, int control)
 	if(addr == 0)
 	{
 		sprintf(buf,"%s%s",path_source,path);
-	
+
 		if((f = fopen(buf,"rb")) == NULL)
 		{
 			sprintf(buf,"%s%s.wav",path_source,path);
@@ -2091,7 +2173,7 @@ static int LoadDataFile(const char *path, int control)
 
 		id = Read4Bytes(f);
 		rewind(f);
-	
+
 		if(id == 0x43455053)
 		{
 			addr = LoadSpect(path, control);
@@ -2121,7 +2203,7 @@ static int LoadDataFile(const char *path, int control)
 			addr = -1;
 		}
 		fclose(f);
-	
+
 		if(addr > 0)
 		{
 			fprintf(f_phcontents,"%c  0x%.5x  %s\n",type_code,addr & 0x7fffff,path);
@@ -2188,7 +2270,7 @@ static int CompileToneSpec(void)
 		*prog_out++ = i_AMPENV + ((amp_env >> 16) & 0xf);
 		*prog_out++ = amp_env;
 	}
-	
+
 	return(0);
 }  // end of CompileToneSpec
 
@@ -2217,7 +2299,7 @@ int CompileSound(int keyword, int isvowel)
 			{
 				value = -128;
 				error("Parameter < -128",NULL);
-			} 
+			}
 		}
 		else
 		{
@@ -2256,9 +2338,11 @@ int CompileIf(int elif)
 	int key;
 	int finish = 0;
 	int word = 0;
+	int word2;
 	int data;
 	int bitmap;
 	int brackets;
+	int not_flag;
 	USHORT *prog_last_if = NULL;
 
 	then_count = 2;
@@ -2266,6 +2350,8 @@ int CompileIf(int elif)
 
 	while(!finish)
 	{
+		not_flag = 0;
+		word2 = 0;
 		if(prog_out >= prog_out_max)
 		{
 			error("Phoneme program too large", NULL);
@@ -2274,10 +2360,25 @@ int CompileIf(int elif)
 
 		if((key = NextItem(tCONDITION)) < 0)
 			error("Expected a condition, not '%s'",item_string);
-	
+
+		if((item_type == 0) && (key == k_NOT))
+		{
+			not_flag = 1;
+			if((key = NextItem(tCONDITION)) < 0)
+				error("Expected a condition, not '%s'",item_string);
+		}
+
 		if(item_type == tWHICH_PHONEME)
 		{
-			// prevPh(), thisPh(), nextPh(), next2Ph()
+			// prevPh(), thisPh(), nextPh(), next2Ph() etc
+			if(key >= 6)
+			{
+			    // put the 'which' code in the next instruction
+			    word2 = key;
+			    key = 6;
+			}
+			key = key << 8;
+
 			data = NextItemBrackets(tPROPERTIES,0);
 			if(data >= 0)
 			{
@@ -2314,12 +2415,20 @@ int CompileIf(int elif)
 		else
 		{
 			error("Unexpected keyword '%s'",item_string);
+
+			if((strcmp(item_string, "phoneme") == 0) || (strcmp(item_string, "endphoneme") == 0))
+				return(-1);
 		}
 
 		// output the word
 		prog_last_if = prog_out;
 		*prog_out++ = word | i_CONDITION;
 
+		if(word2 != 0)
+			*prog_out++ = word2;
+		if(not_flag)
+			*prog_out++ = i_NOT;
+
 		// expect AND, OR, THEN
 		switch(NextItem(tCONDITION))
 		{
@@ -2416,7 +2525,7 @@ int CompileElse(void)
 	{
 		ref = prog_out;
 		*prog_out++ = 0;
-	
+
 		if((p = if_stack[if_level].p_else) != NULL)
 		{
 			*ref = ref - p;    // backwards offset to the previous else
@@ -2461,14 +2570,14 @@ int CompileEndif(void)
 		do
 		{
 			chain = *p;   // a chain of previous else links
-	
+
 			offset = prog_out - p;
 			if(offset > MAX_JUMP)
 			{
 				error("IF block is too long",NULL);
 			}
 			*p = i_JUMP + offset;
-	
+
 			p -= chain;
 		} while (chain > 0);
 	}
@@ -2489,7 +2598,7 @@ static int CompileSwitch(int type)
 		// check the instructions in the Switch
 		return(0);
 	}
-	
+
 //	count_VowelStart = 0;
 //	count_VowelEnding = 0;
 
@@ -2635,7 +2744,6 @@ static void DecThenCount()
 static void InstnPlusPhoneme(int instn)
 {//====================================
 	int phcode;
-	DecThenCount();
 
 	phcode = NextItemBrackets(tPHONEMEMNEM,0);
 	*prog_out++ = instn + phcode;
@@ -2648,11 +2756,15 @@ int CompilePhoneme(int compile_phoneme)
 	int keyword;
 	int value;
 	int phcode = 0;
+	int flags;
 	int ix;
-	unsigned int c;
+	int start;
+	int count;
+	int c;
 	char *p;
+	int vowel_length_factor = 100;  // for testing
 	char number_buf[12];
-	char ipa_buf[N_ITEM_STRING];
+	char ipa_buf[N_ITEM_STRING+1];
 	PHONEME_TAB phoneme_out2;
 	PHONEME_PROG_LOG phoneme_prog_log;
 
@@ -2664,7 +2776,7 @@ int CompilePhoneme(int compile_phoneme)
 	phoneme_flags = 0;
 	place_articulation = 0;
 
-	NextItem(tSTRING); 
+	NextItem(tSTRING);
 	if(compile_phoneme)
 	{
 		phcode = LookupPhoneme(item_string,1);    // declare phoneme if not already there
@@ -2681,6 +2793,8 @@ int CompilePhoneme(int compile_phoneme)
 		}
 		strcpy(proc_names[n_procs], item_string);
 		phoneme_out = &phoneme_out2;
+		sprintf(number_buf,"%.3dP", n_procs);
+		phoneme_out->mnemonic = StringToWord(number_buf);
 	}
 
 	phoneme_out->code = phcode;
@@ -2737,20 +2851,28 @@ int CompilePhoneme(int compile_phoneme)
 			case i_CHANGE_IF+2:
 			case i_CHANGE_IF+3:
 				InstnPlusPhoneme(keyword << 8);
+				DecThenCount();
 				break;
 
 			case i_PAUSE_BEFORE:
-				value = NextItem(tNUMBER);
+				value = NextItemMax(255);
 				*prog_out++ = (i_PAUSE_BEFORE << 8) + value;
+				DecThenCount();
 				break;
 
 			case i_PAUSE_AFTER:
-				value = NextItem(tNUMBER);
+				value = NextItemMax(255);
 				*prog_out++ = (i_PAUSE_AFTER << 8) + value;
+				DecThenCount();
 				break;
 
 			case i_SET_LENGTH:
-				value = NextItem(tNUMBER);
+				value = NextItemMax(511);
+				if(phoneme_out->type == phVOWEL)
+				{
+					value = (value * vowel_length_factor)/100;
+				}
+
 				if(after_if == 0)
 				{
 					phoneme_out->std_length = value/2;
@@ -2763,8 +2885,8 @@ int CompilePhoneme(int compile_phoneme)
 				break;
 
 			case i_ADD_LENGTH:
-				value = NextItem(tSIGNEDNUMBER);
-				*prog_out++ = (i_ADD_LENGTH << 8) + value/2;
+				value = NextItem(tSIGNEDNUMBER) / 2;
+				*prog_out++ = (i_ADD_LENGTH << 8) + (value & 0xff);
 				DecThenCount();
 				break;
 
@@ -2780,33 +2902,60 @@ int CompilePhoneme(int compile_phoneme)
 					strcpy(item_string," ");
 
 				// copy the string, recognize characters in the form U+9999
-				p = item_string;
-				ix = 0;
-				while((c = *p++) != 0)
+				flags = 0;
+				count = 0;
+				ix = 1;
+
+				for(p=item_string; *p != 0;)
 				{
+				    p += utf8_in(&c, p);
+
+				    if((c == '|') && (count > 0))
+				    {
+				        // '|' means don't allow a tie or joiner before this letter
+				        flags |= (1 << (count -1));
+				    }
+				    else
 					if((c=='U') && (p[0]=='+'))
 					{
+					    int j;
 						// U+9999
-						memcpy(number_buf,&p[1],4);  // U+ should be followed by 4 hex digits
+						p++;
+						memcpy(number_buf,p,4);  // U+ should be followed by 4 hex digits
 						number_buf[4] = 0;
-						c = 0;
+						c = '#';
 						sscanf(number_buf,"%x",&c);
-						p += 5;
+
+						// move past the 4 hexdecimal digits
+						for(j=0; j<4; j++)
+						{
+						    if(!isalnum(*p))
+                                break;
+                            p++;
+						}
 						ix += utf8_out(c, &ipa_buf[ix]);
+						count++;
 					}
 					else
 					{
-						ipa_buf[ix++] = c;
+					    ix += utf8_out(c, &ipa_buf[ix]);
+						count++;
 					}
 				}
+				ipa_buf[0] = flags;
 				ipa_buf[ix] = 0;
-				value = strlen(ipa_buf);   // number of UTF-8 bytes
+
+                start = 1;
+                if(flags != 0)
+                    start = 0;   // only include the flags byte if bits are set
+				value = strlen(&ipa_buf[start]);   // number of UTF-8 bytes
 
 				*prog_out++ = (i_IPA_NAME << 8) + value;
 				for(ix=0; ix < value; ix += 2)
 				{
-					*prog_out++ = (ipa_buf[ix] << 8) + (ipa_buf[ix+1] & 0xff);
+					*prog_out++ = (ipa_buf[ix+start] << 8) + (ipa_buf[ix+start+1] & 0xff);
 				}
+				DecThenCount();
 				break;
 			}
 			break;
@@ -2825,7 +2974,7 @@ int CompilePhoneme(int compile_phoneme)
 					phcode = LookupPhoneme(item_string,1);
 				phoneme_out->start_type = phcode;
 				break;
-	
+
 			case kENDTYPE:
 				phcode = NextItem(tPHONEMEMNEM);
 				if(phcode == -1)
@@ -2882,13 +3031,14 @@ int CompilePhoneme(int compile_phoneme)
 			case kSWITCH_PREVVOWEL:
 				endphoneme = CompileSwitch(1);
 				break;
-	
+
 			case kSWITCH_NEXTVOWEL:
 				endphoneme = CompileSwitch(2);
 				break;
 
 			case kCALLPH:
 				CallPhoneme();
+				DecThenCount();
 				break;
 
 			case kFMT:
@@ -2908,7 +3058,7 @@ int CompilePhoneme(int compile_phoneme)
 				DecThenCount();
 				endphoneme = CompileSound(keyword,0);
 				break;
-	
+
 			case kVOWELIN:
 				DecThenCount();
 				endphoneme = CompileVowelTransition(1);
@@ -2920,6 +3070,7 @@ int CompilePhoneme(int compile_phoneme)
 				break;
 
 			case kTONESPEC:
+				DecThenCount();
 				endphoneme = CompileToneSpec();
 				break;
 
@@ -2930,6 +3081,7 @@ int CompilePhoneme(int compile_phoneme)
 
 			case kRETURN:
 				*prog_out++ = i_RETURN;
+				DecThenCount();
 				break;
 
 			case kINCLUDE:
@@ -2968,7 +3120,7 @@ int CompilePhoneme(int compile_phoneme)
 		}
 		phoneme_out->phflags = place_articulation << 16;
 		phoneme_out->phflags |= phoneme_flags;
-	
+
 		if(phoneme_out->phflags & phVOICED)
 		{
 			if(phoneme_out->type == phSTOP)
@@ -2977,15 +3129,15 @@ int CompilePhoneme(int compile_phoneme)
 			if(phoneme_out->type == phFRICATIVE)
 				phoneme_out->type = phVFRICATIVE;
 		}
-	
+
 		if(phoneme_out->std_length == 0)
 		{
 			if(phoneme_out->type == phVOWEL)
 				phoneme_out->std_length = 180/2;  // default length for vowel
 		}
-	
+
 		phoneme_out->phflags |= phLOCAL;  //declared in this phoneme table
-	
+
 		if(phoneme_out->type == phDELETED)
 		{
 			phoneme_out->mnemonic = 0x01;  // will not be recognised
@@ -3287,8 +3439,11 @@ static void CompileEquivalents()
 	p_start[2] = n_bytes >> 8;    // index of next table
 	p_start[3] = n_bytes;
 
-	LoadVoice(voice_name2,0);  // reset the original phoneme table
-	LoadVoiceVariant(save_voice_name,0);
+    if(gui_flag != 0)
+    {
+        LoadVoice(voice_name2,0);  // reset the original phoneme table
+        LoadVoiceVariant(save_voice_name,0);
+    }
 }  // end of CompileEquivalents
 
 
@@ -3325,16 +3480,20 @@ static void CompilePhonemeFiles()
 
 		switch(item)
 		{
+        case kUTF8_BOM:
+            break;  // ignore bytes 0xef 0xbb 0xbf
+
 		case kINCLUDE:
 			NextItem(tSTRING);
 			sprintf(buf,"%s%s",path_source,item_string);
+
 			if((stack_ix < N_STACK) && (f = fopen_log(f_errors,buf,"rb")) != NULL)
 			{
 				fprintf(f_errors,"include %s\n",item_string);
 				stack[stack_ix].linenum = linenum;
 				strcpy(stack[stack_ix].fname,current_fname);
 				stack[stack_ix++].file = f_in;
-				
+
 				f_in = f;
 				strncpy0(current_fname,item_string,sizeof(current_fname));
 				linenum = 1;
@@ -3382,15 +3541,35 @@ static void CompilePhonemeData2(const char *source)
 	wxString report_dict;
 
 #ifdef MAKE_ENVELOPES
-make_envs();	
+make_envs();
 #endif
 
+	wxLogStatus(_T("Compiling phoneme data: ")+wxString(path_source,wxConvLocal));
 	n_envelopes = 0;
 	error_count = 0;
-memset(markers_used,0,sizeof(markers_used));
+	resample_count = 0;
+	memset(markers_used,0,sizeof(markers_used));
 
 	f_errors = stderr;
 
+	if(!wxDirExists(path_phsource))
+	{
+		if(gui_flag)
+		{
+			wxString dirname = wxDirSelector(_T("Phoneme source directory"),path_phsource);
+			if(!dirname.IsEmpty())
+			{
+				path_phsource = dirname;
+				strncpy0(path_source,path_phsource.mb_str(wxConvLocal),sizeof(path_source)-1);
+				strcat(path_source,"/");
+			}
+		}
+		else
+		{
+			fprintf(stderr,"Can't find phoneme source directory: %s\n",path_source);
+		}
+	}
+
 	strncpy0(current_fname,source,sizeof(current_fname));
 
 	strncpy0(fname,path_phfile.mb_str(wxConvLocal),sizeof(fname));
@@ -3430,6 +3609,10 @@ memset(markers_used,0,sizeof(markers_used));
 	if((f_errors = fopen_log(f_errors,fname,"w")) == NULL)
 		f_errors = stderr;
 
+	sprintf(fname,"%s%s",path_source,"compile_report");
+	f_report = fopen_log(f_errors, fname,"w");
+
+
 	sprintf(fname,"%s/%s",path_home,"phondata-manifest");
 	if((f_phcontents = fopen_log(f_phcontents,fname,"w")) == NULL)
 		f_phcontents = stderr;
@@ -3450,6 +3633,11 @@ memset(markers_used,0,sizeof(markers_used));
 "#  -------  ---------\n");
 
 
+	fprintf(f_errors, "Source data path = '%s'\n", path_source);
+	strncpy0(fname,path_phfile.mb_str(wxConvLocal),sizeof(fname));
+	fprintf(f_errors, "Master phonemes file = '%s'\n", fname);
+	fprintf(f_errors, "Output to '%s/'\n\n", path_home);
+
 	sprintf(fname,"%s/%s",path_home,"phondata");
 	f_phdata = fopen_log(f_errors,fname,"wb");
 
@@ -3478,6 +3666,7 @@ memset(markers_used,0,sizeof(markers_used));
 
 	// write a word so that further data doesn't start at displ=0
 	Write4Bytes(f_phdata,version_phdata);
+	Write4Bytes(f_phdata,samplerate_native);
 	Write4Bytes(f_phindex,version_phdata);
 
 	memset(ref_hash_tab,0,sizeof(ref_hash_tab));
@@ -3493,30 +3682,43 @@ memset(markers_used,0,sizeof(markers_used));
 fprintf(f_errors,"\nRefs %d,  Reused %d\n",count_references,duplicate_references);
 	fclose(f_in);
 	fclose(f_phdata);
-	fclose(f_errors);
-	fclose(f_prog_log);
+    if(f_prog_log != NULL)
+        fclose(f_prog_log);
 	fclose(f_phindex);
 	fclose(f_phtab);
 	fclose(f_phcontents);
 
-	LoadPhData();
-	LoadVoice(voice_name2,0);
+	LoadPhData(NULL);
+
+	if(gui_flag != 0)
+        LoadVoice(voice_name2,0);
 
 	CompileReport();
 	report_dict = CompileAllDictionaries();
-//#ifdef deleted
 	DrawEnvelopes();
-//#endif
+
 
 	if(gui_flag)
 	{
 		delete progress;
 	}
 
-	report.Printf(_T("Compiled phonemes: %d errors."),error_count);
+	if(resample_count > 0)
+	{
+	    fprintf(f_errors, "\n%d WAV files resampled to %d Hz\n", resample_count, samplerate_native);
+        report.Printf(_T("Compiled phonemes: %d errors, %d files resampled to %d Hz. "),error_count, resample_count, samplerate_native);
+	}
+	else
+	{
+        report.Printf(_T("Compiled phonemes: %d errors."),error_count);
+	}
+
+	if(f_errors != stderr)
+        fclose(f_errors);
+
 	if(error_count > 0)
 	{
-		report += _T(" See file: 'phsource/error_log'.");
+		report += _T(" See file: '")+wxString(path_source,wxConvLocal)+_T("phsource/error_log'.");
 		wxLogError(report);
 	}
 	wxLogStatus(report + report_dict);
@@ -3525,8 +3727,10 @@ fprintf(f_errors,"\nRefs %d,  Reused %d\n",count_references,duplicate_references
 	{
 		strncpy0(fname,(report+report_dict).mb_str(wxConvLocal),sizeof(fname));
 		fprintf(stderr,"%s\n",fname);
-		
+
 	}
+
+    ReadPhondataManifest();
 }  // end of CompilePhonemeData
 
 
@@ -3605,7 +3809,7 @@ void CompileMbrola()
 		wxLogError(_T("Can't write to: ")+wxString(buf,wxConvLocal));
 		return;
 	}
-	
+
 	data[count].name = 0;  // list terminator
 	Write4Bytes(f_out, mbrola_ctrl);
 
@@ -3959,7 +4163,6 @@ void CompileIntonation()
 	fwrite(tune_data, n_tune_names, sizeof(TUNE), f_out);
 	fclose(f_in);
 	fclose(f_out);
-	fclose(f_errors);
 
 	report.Printf(_T("Compiled %d intonation tunes: %d errors."),n_tune_names, error_count);
 	wxLogStatus(report);
@@ -3968,7 +4171,7 @@ void CompileIntonation()
 	{
 		DisplayErrorFile(fname_errors);
 	}
-	LoadPhData();
+	LoadPhData(NULL);
 
 }   // end of CompileIntonation
 
@@ -3976,7 +4179,26 @@ void CompileIntonation()
 
 void CompilePhonemeData()
 {
+    WavegenInit(22050, 0);
+	WavegenSetVoice(voice);
 	CompilePhonemeData2("phonemes");
-	return;
+}
+
+
+void CompileSampleRate()
+{
+    long value;
+#ifndef PLATFORM_POSIX
+	wxLogError(_T("Change Sample Rate needs the 'sox' program.  It probably doesn't work on Windows"));
+#endif
+
+    value = wxGetNumberFromUser(_T("Compile phoneme data with a specified sample rate"), _T("Sample rate"), _T("Resample (needs 'sox' program)"), 22050, 5000, 48000);
+
+    if(value > 1000)
+    {
+        WavegenInit(value, 0);
+        WavegenSetVoice(voice);
+        CompilePhonemeData2("phonemes");
+    }
 }
 
