@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2010 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2014 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,7 +21,6 @@
 
 
 #include "wx/wfstream.h"
-#include "wx/notebook.h"
 #include "wx/image.h"
 #include "wx/filename.h"
 #include "wx/numdlg.h"
@@ -44,11 +43,11 @@
 #include "prosodydisplay.h"
 
 
-#ifdef deleted
-static const char *about_string = "espeakedit: %s\nAuthor: Jonathan Duddington (c) 2009\n\n"
+
+static const char *about_string2 = "espeakedit: %s\nAuthor: Jonathan Duddington (c) 2009\n\n"
 "Licensed under GNU General Public License version 3\n"
 "http://espeak.sourceforge.net/";
-#endif
+
 
 static const char *about_string = "<font size=0><b>espeakedit </b> %s<br>Author: Jonathan Duddington (c) 2009<br>"
 "<a href=\"http://espeak.sourceforge.net/\">http://espeak.sourceforge.net</a><br>"
@@ -64,6 +63,7 @@ extern void DictionarySort(const char *dictname);
 
 extern void init_z();
 extern void CompilePhonemeData(void);
+extern void CompileSampleRate(void);
 extern void CompileMbrola();
 extern void CompileIntonation();
 extern void InitSpectrumDisplay();
@@ -74,8 +74,6 @@ extern void VowelChart(int control, char *fname);
 extern void MakeVowelLists(void);
 extern void MakeWordFreqList();
 
-extern const char *dict_name;
-
 extern wxMenu *speak_menu;
 extern wxMenu *data_menu;
 
@@ -83,19 +81,16 @@ extern wxMenu *data_menu;
 MyFrame *myframe = NULL;
 SpectDisplay *currentcanvas = NULL;
 
-ChildFrProsody *prosodyframe = NULL;
 ProsodyDisplay *prosodycanvas = NULL;
 wxNotebook *notebook = NULL;
+wxNotebook *screenpages = NULL;
+
 wxProgressDialog *progress;
 int progress_max;
 int gui_flag = 0;
 int frame_x, frame_y, frame_w, frame_h;
 
-wxList my_children;
-
-wxFont FONT_SMALL(8,wxSWISS,wxNORMAL,wxNORMAL);
-wxFont FONT_MEDIUM(9,wxSWISS,wxNORMAL,wxNORMAL);
-wxFont FONT_NORMAL(10,wxSWISS,wxNORMAL,wxNORMAL);
+int adding_page = 0;   // fix for wxWidgets (2,8,7) bug, adding first page to a wxNotebook gives emptystring for GetPageTex() in Notebook_Page_Changed event.
 
 IMPLEMENT_APP(MyApp)
 
@@ -124,7 +119,7 @@ bool MyApp::OnInit(void)
 
 int j;
 wxChar *p;
-char param[80];
+char param[120];
 
 
 if(argc > 1)
@@ -135,22 +130,26 @@ if(argc > 1)
 
 	if((strcmp(param,"--help")==0) || (strcmp(param,"-h")==0))
 	{
-		printf(about_string,espeak_Info(NULL));
-		printf(help_text);
+		printf(about_string2,espeak_Info(NULL));
+		printf("%s", help_text);
 		exit(0);
 	}
 
 	ConfigInit();
-	VoiceReset(0);
-	WavegenSetVoice(voice);
-	WavegenInitSound();
 
 	if(strcmp(param,"--compile")==0)
 	{
+        samplerate_native = samplerate = 22050;
+	    LoadPhData(NULL);
+		if(LoadVoice("", 0) == NULL)
+		{
+			fprintf(stderr, "Failed to load default voice\n");
+			exit(1);
+		}
 		CompilePhonemeData();
 		CompileIntonation();
-		exit(0);
 	}
+    exit(0);
 }
 
 	ConfigInit();
@@ -166,7 +165,7 @@ if(argc > 1)
 
 	if((frame_w == 0) || (frame_h == 0))
 	{
-		frame_w = 1024;
+		frame_w = 800;
 		frame_h = 768;
 	}
 
@@ -188,32 +187,35 @@ if(argc > 1)
 	SetTopWindow(myframe);
 	wxInitAllImageHandlers();
 //	wxImage::AddHandler(wxPNGHandler);
+	wxLogStatus(_T("Using espeak_data at: ")+wxString(path_home, wxConvLocal));
   return TRUE;
 }
 
-BEGIN_EVENT_TABLE(MyFrame, wxMDIParentFrame)
+BEGIN_EVENT_TABLE(MyFrame, wxFrame)
 	EVT_CHAR(MyFrame::OnKey)
    EVT_MENU(MENU_ABOUT, MyFrame::OnAbout)
-	EVT_MENU(MENU_DOCS, MyFrame::OnAbout)
+   EVT_MENU(MENU_DOCS, MyFrame::OnAbout)
    EVT_MENU(MENU_SPECTRUM, MyFrame::OnNewWindow)
    EVT_MENU(MENU_SPECTRUM2, MyFrame::OnNewWindow)
    EVT_MENU(MENU_PROSODY, MyFrame::OnProsody)
    EVT_MENU(MENU_OPT_SPEED, MyFrame::OnOptions)
    EVT_MENU(MENU_OPT_PUNCT, MyFrame::OnOptions)
    EVT_MENU(MENU_OPT_SPELL, MyFrame::OnOptions)
+   EVT_MENU(MENU_OPT_SPELL2, MyFrame::OnOptions)
+   EVT_MENU(MENU_PATH_DATA, MyFrame::OnOptions)
    EVT_MENU(MENU_PATH0, MyFrame::OnOptions)
    EVT_MENU(MENU_PATH1, MyFrame::OnOptions)
    EVT_MENU(MENU_PATH2, MyFrame::OnOptions)
    EVT_MENU(MENU_PATH3, MyFrame::OnOptions)
    EVT_MENU(MENU_PATH4, MyFrame::OnOptions)
    EVT_MENU(MENU_COMPILE_PH, MyFrame::OnTools)
+   EVT_MENU(MENU_COMPILE_PH2, MyFrame::OnTools)
 	EVT_MENU(MENU_COMPILE_DICT, MyFrame::OnTools)
 	EVT_MENU(MENU_COMPILE_DICT_DEBUG, MyFrame::OnTools)
 	EVT_MENU(MENU_FORMAT_DICTIONARY, MyFrame::OnTools)
 	EVT_MENU(MENU_SORT_DICTIONARY, MyFrame::OnTools)
 	EVT_MENU(MENU_COMPILE_MBROLA, MyFrame::OnTools)
 	EVT_MENU(MENU_COMPILE_INTONATION, MyFrame::OnTools)
-	EVT_MENU(MENU_CLOSE_ALL, MyFrame::OnQuit)
 	EVT_MENU(MENU_QUIT, MyFrame::OnQuit)
 	EVT_MENU(MENU_SPEAK_TRANSLATE, MyFrame::OnSpeak)
 	EVT_MENU(MENU_SPEAK_RULES, MyFrame::OnSpeak)
@@ -238,14 +240,22 @@ BEGIN_EVENT_TABLE(MyFrame, wxMDIParentFrame)
 	EVT_MENU(MENU_TEST, MyFrame::OnTools)
 	EVT_MENU(MENU_TEST2, MyFrame::OnTools)
 
+    EVT_MENU(SPECTSEQ_SAVE, MyFrame::PageCmd)
+    EVT_MENU(SPECTSEQ_SAVEAS, MyFrame::PageCmd)
+    EVT_MENU(SPECTSEQ_SAVESELECT, MyFrame::PageCmd)
+    EVT_MENU(SPECTSEQ_CLOSE, MyFrame::PageCmd)
+    EVT_MENU(SPECTSEQ_SAVEPITCH, MyFrame::PageCmd)
+	EVT_MENU(MENU_CLOSE_ALL, MyFrame::PageCmd)
+
+
+    EVT_NOTEBOOK_PAGE_CHANGED(ID_SCREENPAGES, MyFrame::OnPageChanged)
 	EVT_TIMER(1, MyFrame::OnTimer)
-	EVT_SIZE(MyFrame::OnSize)
-	EVT_SASH_DRAGGED(ID_WINDOW_LEFT, MyFrame::OnSashDrag)
 END_EVENT_TABLE()
 
 
 MyFrame::~MyFrame(void)
 {//====================
+	myframe->Maximize(false);
 	myframe->Show(false);
 	myframe->Iconize(false);   // os=Windows, get the non-iconsized size
 	myframe->GetPosition(&frame_x, &frame_y);
@@ -254,24 +264,16 @@ MyFrame::~MyFrame(void)
 
 MyFrame::MyFrame(wxWindow *parent, const wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size,
     const long style):
-  wxMDIParentFrame(parent, id, title, pos, size, style)
+  wxFrame(parent, id, title, pos, size, style)
 {//===================================================================================================================
 // Main Frame constructor
 
 	int error_flag = 0;
 	int result;
 	int param;
+	int srate;
 
-  // Another window to the left of the client window
-  m_leftWindow = new wxSashLayoutWindow(this, ID_WINDOW_LEFT,
-                               wxDefaultPosition, wxSize(298, 30),
-                               wxNO_BORDER | wxSW_3D | wxCLIP_CHILDREN);
-  m_leftWindow->SetDefaultSize(wxSize(310, 1000));
-  m_leftWindow->SetOrientation(wxLAYOUT_VERTICAL);
-  m_leftWindow->SetAlignment(wxLAYOUT_LEFT);
-  m_leftWindow->SetSashVisible(wxSASH_RIGHT, TRUE);
-
-	notebook = new wxNotebook(m_leftWindow,-1);
+	notebook = new wxNotebook(this, ID_NOTEBOOK, wxDefaultPosition, wxSize(312,760));
 //	notebook->AddPage(voicedlg,_T("Voice"),FALSE);
 	formantdlg = new FormantDlg(notebook);
 	notebook->AddPage(formantdlg,_T(" Spect"),FALSE);
@@ -281,10 +283,32 @@ MyFrame::MyFrame(wxWindow *parent, const wxWindowID id, const wxString& title, c
 	notebook->AddPage(transldlg,_T("Text"),TRUE);
 
 
-	LoadConfig();
-	WavegenInitSound();
+    screenpages = new wxNotebook(this, ID_SCREENPAGES, wxDefaultPosition, wxSize(554,702));
 
-	if((result = LoadPhData()) != 1)
+    wxBoxSizer *framesizer = new wxBoxSizer( wxHORIZONTAL );
+
+
+    framesizer->Add(
+        notebook,
+        0,            // make horizontally stretchable
+        wxEXPAND |    // make vertically stretchable
+        wxALL,        //   and make border all around
+        4 );         // set border width
+
+    framesizer->Add(
+        screenpages,
+        1,            // make horizontally stretchable
+        wxEXPAND |    // make vertically stretchable
+        wxALL,        //   and make border all around
+        4 );         // set border width
+
+    SetSizer( framesizer );      // use the sizer for layout
+    framesizer->SetSizeHints( this );   // set size hints to honour minimum size
+    SetSize(pos.x, pos.y, size.GetWidth(), size.GetHeight());
+
+	LoadConfig();
+
+	if((result = LoadPhData(&srate)) != 1)
 	{
 		if(result == -1)
 			wxLogError(_T("Failed to read espeak-data/phontab,phondata,phonindex\nPath = ")+wxString(path_home,wxConvLocal)+_T("\n\nThe 'eSpeak' package needs to be installed"));
@@ -292,7 +316,10 @@ MyFrame::MyFrame(wxWindow *parent, const wxWindowID id, const wxString& title, c
 			wxLogError(_T("Wrong version of espeak-data at:\n")+ wxString(path_home,wxConvLocal)+_T("\nVersion 0x%x (expects 0x%x)"),result,version_phdata);
 
 		error_flag = 1;
+		srate = 22050;
 	}
+	WavegenInit(srate,0);
+	WavegenInitSound();
 
 	f_trans = stdout;
 	option_ssml = 1;
@@ -310,7 +337,7 @@ MyFrame::MyFrame(wxWindow *parent, const wxWindowID id, const wxString& title, c
 
 	for(param=0; param<N_SPEECH_PARAM; param++)
 		param_stack[0].parameter[param] = param_defaults[param];
-	
+
 	SetParameter(espeakRATE,option_speed,0);
 
 	SetSpeed(3);
@@ -331,7 +358,13 @@ MyFrame::MyFrame(wxWindow *parent, const wxWindowID id, const wxString& title, c
 void MyFrame::SetVoiceTitle(char *voice_name)
 {//==========================================
 	char buf[100];
-	SetTitle(AppName + _T(" - ") + wxString(voice_name,wxConvLocal) + _T("  voice"));
+
+	if(samplerate_native == 22050)
+        sprintf(buf, " - %s  voice", voice_name);
+    else
+        sprintf(buf, " - %s  voice  %dHz", voice_name, samplerate_native);
+	SetTitle(AppName + wxString(buf,wxConvLocal));
+
 	if((data_menu != NULL) && (translator != NULL))
 	{
 		sprintf(buf,"Compile &dictionary '%s'",translator->dictionary_name);
@@ -345,6 +378,111 @@ void MyFrame::SetVoiceTitle(char *voice_name)
 
 
 
+void MyFrame::PageCmd(wxCommandEvent& event)
+{//=========================================
+    int pagenum;
+    int ix;
+    int n_pages;
+    SpectDisplay *page;
+
+//    if(currentcanvas != NULL)
+    {
+        pagenum = screenpages->GetSelection();
+
+        switch(event.GetId())
+        {
+        case SPECTSEQ_SAVE:
+            currentcanvas->Save(currentcanvas->savepath);
+            break;
+        case SPECTSEQ_SAVEAS:
+            currentcanvas->Save();
+            screenpages->SetPageText(screenpages->GetSelection(), currentcanvas->spectseq->name+_T(" ²"));
+            break;
+        case SPECTSEQ_SAVESELECT:
+            currentcanvas->Save(_T(""), 1);
+            break;
+        case SPECTSEQ_CLOSE:
+            if(screenpages->GetPageText(pagenum) != _T("Prosody"))
+            {
+                currentcanvas->OnActivate(0);
+            }
+            screenpages->DeletePage(pagenum);
+
+            if((n_pages = screenpages->GetPageCount()) > 0)
+            {
+                if(pagenum >= n_pages)
+                    pagenum--;
+                page = (SpectDisplay *)screenpages->GetPage(pagenum);
+
+                if(screenpages->GetPageText(pagenum) == _T("Prosody"))
+                {
+                    MakeMenu(3, NULL);
+                }
+                else
+                {
+                    page->OnActivate(1);
+                    MakeMenu(2, NULL);
+                }
+            }
+            else
+            {
+                MakeMenu(1, NULL);
+            }
+            break;
+
+        case MENU_CLOSE_ALL:
+            n_pages = screenpages->GetPageCount();
+            for(ix=n_pages-1; ix>=0; ix--)
+            {
+                screenpages->DeletePage(ix);
+            }
+            currentcanvas = NULL;
+            MakeMenu(1, NULL);
+            break;
+        case SPECTSEQ_SAVEPITCH:
+            currentcanvas->SavePitchenv(currentcanvas->spectseq->pitchenv);
+            break;
+        }
+    }
+}
+
+
+
+void MyFrame::OnPageChanged(wxNotebookEvent& event)
+{//=================================================
+    int pagenum;
+    wxString title;
+    SpectDisplay *page;
+
+    pagenum = event.GetSelection();
+
+    if(event.GetId() == ID_SCREENPAGES)
+    {
+        title = screenpages->GetPageText(pagenum);
+
+        if((title != _T("Prosody")) && (adding_page != 2))
+        {
+            page = (SpectDisplay *)screenpages->GetPage(pagenum);
+
+            if(page != currentcanvas)
+            {
+                if(currentcanvas != NULL)
+                {
+                    currentcanvas->OnActivate(0);
+                }
+                page->OnActivate(1);
+            }
+            MakeMenu(2, NULL);
+        }
+        else
+        {
+            MakeMenu(3, NULL);
+        }
+    }
+	adding_page = 0;   // work around for wxNotebook bug (version 2.8.7)
+}
+
+
 void MyFrame::OnKey(wxKeyEvent& event)
 {
 	int key;
@@ -492,14 +630,34 @@ void OnOptions2(int event_id)
 		break;
 
 	case MENU_OPT_SPELL:
+		transldlg->t_source->SetValue(_T("<say-as interpret-as=\"characters\">\n"));
+		transldlg->t_source->SetInsertionPointEnd();
+		notebook->SetSelection(1);
+		break;
+
+	case MENU_OPT_SPELL2:
 		transldlg->t_source->SetValue(_T("<say-as interpret-as=\"tts:char\">\n"));
 		transldlg->t_source->SetInsertionPointEnd();
 		notebook->SetSelection(1);
 		break;
 
+	case MENU_PATH_DATA:
+		string = wxDirSelector(_T("espeak_data directory"), wxEmptyString);
+		if(!string.IsEmpty())
+		{
+			if(!wxDirExists(string+_T("/voices")))
+			{
+				wxLogError(_T("No 'voices' directory in ") + string);
+				break;
+			}
+			path_espeakdata = string;
+			wxLogMessage(_T("Quit and restart espeakedit to use the new espeak_data location"));
+		}
+		break;
+
 	case MENU_PATH0:
 		string = wxFileSelector(_T("Master phonemes file"),wxFileName(path_phfile).GetPath(),
-			_T(""),_T(""),_T("*"),wxOPEN);
+			_T("phonemes"),_T(""),_T("*"),wxOPEN);
 		if(!string.IsEmpty())
 		{
 			path_phfile = string;
@@ -562,7 +720,7 @@ void DisplayErrorFile(const char *fname)
 	if(len > 0)
 	{
 		if(len > 1500)
-			len = 1500;   // restrict length to pppppevent crash in wxLogMessage()
+			len = 1500;   // restrict length to prevent crash in wxLogMessage()
 		msg = (char *)malloc(len+1);
 		if(msg != NULL)
 		{
@@ -586,6 +744,7 @@ void MyFrame::OnTools(wxCommandEvent& event)
 	int debug_flag=0;
 	char fname_log[sizeof(path_dsource)+12];
 	char err_fname[sizeof(path_home)+15];
+	static const char utf8_bom[] = {0xef,0xbb,0xbf,0};
 
 	switch(event.GetId())
 	{
@@ -615,6 +774,12 @@ void MyFrame::OnTools(wxCommandEvent& event)
 
 	case MENU_COMPILE_PH:
 		CompilePhonemeData();
+		SetVoiceTitle(voice_name2);
+		break;
+
+	case MENU_COMPILE_PH2:
+		CompileSampleRate();
+		SetVoiceTitle(voice_name2);
 		break;
 
 	case MENU_COMPILE_MBROLA:
@@ -624,12 +789,16 @@ void MyFrame::OnTools(wxCommandEvent& event)
 	case MENU_COMPILE_INTONATION:
 		CompileIntonation();
 		break;
-		
+
 	case MENU_COMPILE_DICT_DEBUG:
 		debug_flag =1;  // and drop through to next case
 	case MENU_COMPILE_DICT:
 		sprintf(fname_log,"%s%s",path_dsource,"dict_log");
 		log = fopen(fname_log,"w");
+		if(log != NULL)
+		{
+			fprintf(log, "%s", utf8_bom);
+		}
 
 		LoadDictionary(translator, translator->dictionary_name, 0);
 		if((err = CompileDictionary(path_dsource,translator->dictionary_name,log,err_fname,debug_flag)) < 0)
@@ -729,105 +898,4 @@ void MyFrame::OnSpeak(wxCommandEvent& event)
 }
 
 
-void MyFrame::OnSashDrag(wxSashEvent& event)
-{
-	int w, h;
-
-	if (event.GetDragStatus() == wxSASH_STATUS_OUT_OF_RANGE)
-		return;
-
-	GetClientSize(&w, &h);
-
-	switch (event.GetId())
-	{
-		case ID_WINDOW_LEFT:
-		{
-			m_leftWindow->SetDefaultSize(wxSize(event.GetDragRect().width, h));
-			break;
-		}
-	}
-	wxLayoutAlgorithm layout;
-	layout.LayoutMDIFrame(this);
-
-	// Leaves bits of itself behind sometimes
-	GetClientWindow()->Refresh();
-}
-
-
-
-void MyFrame::OnSize(wxSizeEvent& WXUNUSED(event))
-{
-    wxLayoutAlgorithm layout;
-    layout.LayoutMDIFrame(this);
-}
-
-// Note that SASHTEST_NEW_WINDOW and SASHTEST_ABOUT commands get passed
-// to the parent window for processing, so no need to
-// duplicate event handlers here.
-
-BEGIN_EVENT_TABLE(MyChild, wxMDIChildFrame)
-  EVT_MENU(SPECTSEQ_CLOSE, MyChild::OnQuit)
-  EVT_MENU(SPECTSEQ_SAVE, MyChild::OnSave)
-  EVT_MENU(SPECTSEQ_SAVEAS, MyChild::OnSaveAs)
-  EVT_MENU(SPECTSEQ_SAVESELECT, MyChild::OnSaveSelect)
-  EVT_MENU(SPECTSEQ_SAVEPITCH, MyChild::OnSavePitchenv)
-
-  EVT_ACTIVATE(MyChild::OnActivate)
-END_EVENT_TABLE()
-
-MyChild::MyChild(wxMDIParentFrame *parent, const wxString& title, const wxPoint& pos, const wxSize& size,
-const long style):
-  wxMDIChildFrame(parent, -1, title, pos, size, style)
-{
-  canvas = NULL;
-  my_children.Append(this);
-}
-
-MyChild::~MyChild(void)
-{
-	canvas = NULL;
-#ifndef PLATFORM_WINDOWS
-	wxWindow *w;
-	// bug in wxMDIChildFrame, we need to explicitly remove the ChildFrame from the ClientWindow
-	w = myframe->GetClientWindow();
-	w->RemoveChild(this);
-#endif
-  my_children.DeleteObject(this);
-}
-
-//extern void CloseCanvas();
-
-void MyChild::OnQuit(wxCommandEvent& WXUNUSED(event))
-{
-	Destroy();
-
-}
-
-void MyChild::OnSave(wxCommandEvent& WXUNUSED(event))
-{
-	canvas->Save(canvas->savepath);
-}
-
-void MyChild::OnSaveAs(wxCommandEvent& WXUNUSED(event))
-{
-	canvas->Save();
-}
-
-void MyChild::OnSaveSelect(wxCommandEvent& WXUNUSED(event))
-{
-	canvas->Save(_T(""),1);
-}
-
-void MyChild::OnSavePitchenv(wxCommandEvent& WXUNUSED(event))
-{
-	canvas->SavePitchenv(canvas->spectseq->pitchenv);
-}
-
-void MyChild::OnActivate(wxActivateEvent& event)
-{
-	if(canvas)
-		canvas->OnActivate(event.GetActive());
-
-		
-}
 
