@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2014 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -31,7 +31,6 @@
 #include "translate.h"
 
 extern MyFrame *myframe;
-extern ChildFrProsody *prosodyframe;
 extern ProsodyDisplay *prosodycanvas;
 
 wxMenu *menu_prosody;
@@ -51,6 +50,86 @@ static wxPen PEN_PHSELECTED(wxColour(255,0,0),2,wxSOLID);
 static wxPen PEN_PHSTRESSED(wxColour(80,80,196),3,wxSOLID);
 static wxPen PEN_PHSTRESSED2(wxColour(160,160,255),2,wxSOLID);
 
+
+
+
+typedef struct {
+    unsigned int value;
+    char *name;
+} NAMETAB;
+
+NAMETAB *manifest = NULL;
+int n_manifest;
+
+const char *LookupManifest(unsigned int addr)
+{//=============================================
+    int ix;
+
+    if(manifest != NULL)
+    {
+        for(ix=0; ix < n_manifest; ix++)
+        {
+            if(manifest[ix].value == addr)
+                return(manifest[ix].name);
+            if(manifest[ix].value > addr)
+                break;
+        }
+    }
+    return("");
+}
+
+
+void ReadPhondataManifest()
+{//=========================
+// Read the phondata-manifest file
+    FILE *f;
+    int n_lines=0;
+    int ix;
+    char *p;
+    unsigned int value;
+    char buf[sizeof(path_home)+40];
+    char name[120];
+
+	sprintf(buf,"%s%c%s",path_home,PATHSEP,"phondata-manifest");
+    if((f = fopen(buf, "r")) == NULL)
+        return;
+
+    while(fgets(buf, sizeof(buf), f) != NULL)
+        n_lines++;
+
+    rewind(f);
+
+    if(manifest != NULL)
+    {
+        for(ix=0; ix < n_manifest; ix++)
+            free(manifest[ix].name);
+    }
+
+    if((manifest = (NAMETAB *)realloc(manifest, n_lines * sizeof(NAMETAB))) == NULL)
+        return;
+
+    n_manifest = 0;
+    while(fgets(buf, sizeof(buf), f) != NULL)
+    {
+        if(!isalpha(buf[0]))
+           continue;
+
+        if(sscanf(&buf[2], "%x %s", &value, name) == 2)
+        {
+            if((p = (char *)malloc(strlen(name)+1)) != NULL)
+            {
+                strcpy(p, name);
+                manifest[n_manifest].value = value;
+                manifest[n_manifest].name = p;
+                n_manifest++;
+            }
+        }
+    }
+    fclose(f);
+}
+
+
+
 ProsodyDisplay::ProsodyDisplay(wxWindow *parent, const wxPoint& pos, const wxSize& size)
         : wxScrolledWindow(parent, -1, pos, size,
                            wxSUNKEN_BORDER | wxNO_FULL_REPAINT_ON_RESIZE)
@@ -60,6 +139,7 @@ ProsodyDisplay::ProsodyDisplay(wxWindow *parent, const wxPoint& pos, const wxSiz
 	scalex = 0.5;
 	scaley = double(LINESEP*6)/150.0;
 	selected_ph = -1;
+	SetBackgroundColour(wxColour(245,245,245));
 }  // end of ProsodyDisplay::ProsodyDisplay
 
 
@@ -76,6 +156,7 @@ void InitProsodyDisplay()
 	int ix;
 	wxString string;
 
+    ReadPhondataManifest();
 	menu_envelopes = new wxMenu;
 	// entries match those in envelope_data[] in intonation.cpp
 
@@ -119,7 +200,7 @@ void ProsodyDisplay::RefreshLine(int line)
 int ProsodyDisplay::GetWidth(PHONEME_LIST *p)
 {//========================================
 	int  w;
-	
+
 	if(p->ph == NULL)
 		return(0);
 
@@ -142,6 +223,8 @@ void ProsodyDisplay::SelectPh(int index)
 	int ix;
 	const char *name = "?";
 	char buf[120];
+	char len_string[20];
+	char param_string[20];
 
 	if(index < 0) return;
 
@@ -162,8 +245,15 @@ void ProsodyDisplay::SelectPh(int index)
 	}
 	y1 = p->pitch1;
 	y2 = p->pitch2;
-	sprintf(buf,"Stress %s%d   Amp %2d   LenMod %2d   Pitch %3d %3d  %s    PhFlags %.2x ",
-		emphasized,p->stresslevel&0x7,p->amp,p->length,y1,y2,name,p->ph->phflags);
+	len_string[0] = 0;
+	param_string[0] = 0;
+	if(p->std_length > 0)
+        sprintf(len_string,"  Length %d", p->std_length*2);
+	if(p->sound_param != 0)
+		sprintf(param_string,", %d", p->sound_param);
+
+	sprintf(buf,"Stress %s%d   Amp %2d   LengthMod %2d   Pitch %3d %3d  %s    PhFlags %.2x    (%s%s)%s",
+		emphasized,p->stresslevel&0x7,p->amp, p->length,y1,y2,name,p->ph->phflags, LookupManifest(p->phontab_addr), param_string, len_string);
 	wxLogStatus(wxString(buf,wxConvLocal));
 }
 
@@ -253,19 +343,22 @@ void ProsodyDisplay::OnMouse(wxMouseEvent& event)
 
 	line = pt.y / FRAMEHEIGHT;
 
-	// find which phoneme is selected on this line
-	for(ix=linetab[line]; (ix<linetab[line+1]) && (ix<numph); ix++)
+	if(line < num_lines)
 	{
-		xpos += GetWidth(&phlist[ix]);
-		if(xpos > pt.x)
+		// find which phoneme is selected on this line
+		for(ix=linetab[line]; (ix<linetab[line+1]) && (ix<numph); ix++)
 		{
-			selected_ph = ix;
-			SelectPh(selected_ph);
-			break;
+			xpos += GetWidth(&phlist[ix]);
+			if(xpos > pt.x)
+			{
+				selected_ph = ix;
+				SelectPh(selected_ph);
+				break;
+			}
 		}
-	}
 
-	RefreshLine(line);
+		RefreshLine(line);
+	}
 
 	if(event.RightDown())
 	{
@@ -279,7 +372,7 @@ void ProsodyDisplay::OnKey(wxKeyEvent& event)
 {//========================================
 	PHONEME_LIST *p;
 	int display=1;
-	
+
 	if(selected_ph < 0)
 		selected_ph = 0;
 
@@ -301,7 +394,7 @@ void ProsodyDisplay::OnKey(wxKeyEvent& event)
 		if(selected_ph < (numph-2))
 			selected_ph++;
 		break;
-		
+
 	case WXK_UP:
 		if(event.ControlDown())
 			ChangePh(-1,2);
@@ -360,7 +453,7 @@ void ProsodyDisplay::DrawEnv(wxDC& dc, int x1, int y1, int width, PHONEME_LIST *
 	PHONEME_DATA phdata_tone;
 
 	if(width <= 0) return;
-	
+
 	if((pitchr = ph->pitch2 - ph->pitch1) < 0)
 	{
 		pitchr = -pitchr;
@@ -425,6 +518,7 @@ void ProsodyDisplay::DrawPitchline(wxDC& dc, int line, int x1, int x2)
 
 	y = LINEBASE+LINESEP;
 	dc.SetPen(*wxLIGHT_GREY_PEN);
+//	dc.SetPen(*wxCYAN_PEN);
 	for(ix=0; ix<5; ix++)
 	{
 		dc.DrawLine(0,offy-y,linewidth,offy-y);
@@ -520,6 +614,12 @@ void ProsodyDisplay::OnDraw(wxDC& dc)
 
 	int line, start, end;
 
+    GetClientSize(&x1, &y1);
+    if(x1 != linewidth)
+    {
+        LayoutData(NULL, 0);
+    }
+
 	wxRegionIterator upd(GetUpdateRegion()); // get the update rect list
 
 	while (upd)
@@ -549,15 +649,18 @@ void ProsodyDisplay::LayoutData(PHONEME_LIST *ph_list, int n_ph)
 	int xpos;
 	int w;
 	int ix;
+	int height;
 	PHONEME_LIST *p;
 
-
-
-	numph = n_ph;
-	phlist = ph_list;
+    if(ph_list != NULL)
+    {
+        numph = n_ph;
+        phlist = ph_list;
+    }
 
 	num_lines = 0;
 	linetab[0] = 1;
+	GetClientSize(&linewidth, &height);
 	xpos = linewidth;
 
 	// could improve this to do 'wordwrap' - only split on word boundary
@@ -584,86 +687,34 @@ void ProsodyDisplay::LayoutData(PHONEME_LIST *ph_list, int n_ph)
 }  // end of ProsodyDisplay::LayoutData
 
 
+extern int adding_page;
 
 void MyFrame::OnProsody(wxCommandEvent& WXUNUSED(event))
 {//=====================================================
-
 	// Open the Prosody display window
-	// Make another frame, containing a canvas
+	int width, height;
+	int ix, npages;
 
-	int h, w, w2;
-
-	if(prosodyframe != NULL)
+	if(prosodycanvas != NULL)
 	{
 		// The Prosody window is already open
-		prosodyframe->Activate();
+
+		// ?? select the prosody page ??
+		npages = screenpages->GetPageCount();
+		for(ix=0; ix<npages; ix++)
+		{
+		    if(screenpages->GetPage(ix) == (wxWindow*)prosodycanvas)
+		    {
+		        screenpages->ChangeSelection(ix);
+		        break;
+		    }
+		}
 		return;
 	}
 
-	m_leftWindow->GetSize(&w2, &h);
-	GetClientSize(&w, &h);
-	prosodyframe = new ChildFrProsody(myframe, _T(""),
-                                      wxPoint(0, 100), wxSize(w-w2, 420),
-                                      wxDEFAULT_FRAME_STYLE |
-                                      wxNO_FULL_REPAINT_ON_RESIZE);
-
-	prosodyframe->SetTitle(_T("Prosody"));
-
-	// Give it a status line
-	prosodyframe->CreateStatusBar();
-
-	int width, height;
-	wxMDIClientWindow *clientwin = (wxMDIClientWindow *)this->GetClientWindow();
-	clientwin->GetClientSize(&width, &height);
-
-#ifdef deleted
-	wxPanel *panel = new wxPanel(prosodyframe,-1,wxPoint(0,0), wxSize(width,50));
-	ProsodyDisplay *canvas = new ProsodyDisplay(prosodyframe, wxPoint(0, 50), wxSize(width-2, height-50));
-#else
-	ProsodyDisplay *canvas = new ProsodyDisplay(prosodyframe, wxPoint(0, 50), wxSize(width-10, height));
-#endif
-
-	prosodycanvas = canvas;
-	
-	// Associate the menu bar with the frame
-	prosodyframe->SetMenuBar(MakeMenu(2,translator->dictionary_name));
-	prosodyframe->prosodycanvas = canvas;
-	prosodyframe->Show(TRUE);
 
+    screenpages->GetClientSize(&width, &height);
+	prosodycanvas = new ProsodyDisplay(screenpages, wxPoint(0, 50), wxSize(width-10, height));
+	adding_page = 2;  // work around for wxNotebook bug (version 2.8.7)
+	screenpages->AddPage(prosodycanvas, _T("Prosody"), true);
 }
-
-BEGIN_EVENT_TABLE(ChildFrProsody, wxMDIChildFrame)
-  EVT_MENU(SPECTSEQ_CLOSE, ChildFrProsody::OnQuit)
-
-//  EVT_ACTIVATE(ChildFrProsody::OnActivate)
-END_EVENT_TABLE()
-
-extern wxList my_children;
-
-ChildFrProsody::ChildFrProsody(wxMDIParentFrame *parent, const wxString& title, const wxPoint& pos, const wxSize& size,
-const long style):
-  wxMDIChildFrame(parent, -1, title, pos, size, style)
-{
-	my_children.Append(this);
-}
-
-ChildFrProsody::~ChildFrProsody(void)
-{
-	wxWindow *w;
-	my_children.DeleteObject(this);
-	prosodycanvas = NULL;
-	prosodyframe = NULL;
-
-#ifndef PLATFORM_WINDOWS
-	// bug in wxMDIChildFrame, we need to explicitly remove the ChildFrame from the ClientWindow
-	w = myframe->GetClientWindow();
-	w->RemoveChild(this);
-#endif
-}
-
-void ChildFrProsody::OnQuit(wxCommandEvent& WXUNUSED(event))
-{
-      Close(TRUE);
-}
-
-
