@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2013 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -325,12 +325,12 @@ void TranslDlg::SpeakFile(void)
 	wxFileName fname;
 	FILE *f_text;
 	char buf[200];
-	
+
 	fname = wxFileName(path_speaktext);
 
 	file = wxFileSelector(_T("Text file to speak"),fname.GetPath(),fname.GetName(),_T(""),_T("*"),wxOPEN);
 	if(file == wxEmptyString) return;
-	
+
 	strcpy(buf,file.mb_str(wxConvLocal));
 	f_text = fopen(buf,"r");
 	if(f_text == NULL)
@@ -338,9 +338,9 @@ void TranslDlg::SpeakFile(void)
 		wxLogError(_T("Failed to read: ")+file);
 		return;
 	}
-	
+
 	path_speaktext = file;
-	
+
 	InitText(0);
 	SpeakNextClause(f_text,NULL,0);
 	return;
@@ -354,7 +354,7 @@ void TranslDlg::ReadVoice(int variant)
 	wxString filename;
 	char *p;
 	char vname[40];
-	char fname[sizeof(path_home)+10];
+	char fname[sizeof(path_home)+30];
 
 	if(variant)
 	{
@@ -406,6 +406,7 @@ void TranslDlg::OnCommand(wxCommandEvent& event)
 	int translate_text = 0;
 	char buf[1000];
 	char phon_out[N_PH_LIST*2];
+	const char *phon_out2;
 	int clause_tone;
 	int clause_count;
 	int use_ipa = 0;
@@ -460,16 +461,14 @@ void TranslDlg::OnCommand(wxCommandEvent& event)
 		t_phonetic->SetDefaultStyle(style_phonetic_large);
 
 		translate_text = 3;
-		use_ipa = 1;
+		use_ipa = 0x10;
 		break;
 
 	case T_PROCESS:
 	case MENU_SPEAK_TEXT:
-		if(prosodycanvas == NULL)
-		{
-			myframe->OnProsody(event);
-		}
+        myframe->OnProsody(event);
 		prosodycanvas->LayoutData(ph_list,n_ph_list);
+        myframe->Refresh();
 		option_phoneme_events = espeakINITIALIZE_PHONEME_EVENTS;
 		option_log_frames = 1;
 		MakeWave2(ph_list,n_ph_list);
@@ -497,21 +496,23 @@ void TranslDlg::OnCommand(wxCommandEvent& event)
 			CalcPitches(translator,clause_tone);
 			CalcLengths(translator);
 
-			GetTranslatedPhonemeString(translator->phon_out, sizeof(translator->phon_out), use_ipa);
+			phon_out2 = GetTranslatedPhonemeString(use_ipa);
 			if(clause_count++ > 0)
 				strcat(phon_out," ||");
-			strcat(phon_out,translator->phon_out);
-			t_phonetic->SetValue(wxString(translator->phon_out,wxConvUTF8));
+			strcat(phon_out, phon_out2);
+			t_phonetic->SetValue(wxString(phon_out2, wxConvUTF8));
 
 			if((n_ph_list + n_phoneme_list) >= N_PH_LIST)
 			{
-				n_phoneme_list = N_PH_LIST - n_ph_list - n_phoneme_list;
+				n_phoneme_list = N_PH_LIST - n_ph_list;
 			}
 
 			memcpy(&ph_list[n_ph_list],phoneme_list,sizeof(PHONEME_LIST)*n_phoneme_list);
 			n_ph_list += n_phoneme_list;
 		}
 
+        ph_list[N_PH_LIST].ph = NULL;  // to recognize overrun off list (in Generate() )
+
 		t_phonetic->Clear();
 		if(option_phonemes == 2)
 		{
